// Draggable (jQuery) Plugin
/*! Copyright (c) 2013 Elton Jain
 *
 * Version: 0.1
 *
 * Requires: jQuery 1.7.2 or higher
 */

(function($) {

    var active = null;
    var NORMAL_Z_INDEX = 1000;
    var ACTIVE_Z_INDEX = 10000;

    $.fn.myDraggable = function (opt) {
        var $el = this;
       

        var defaultSettings = {
            handle: "",
            cursor: "move",
            checkCondition: null,
            axis: null,
            insideParentContainer: false,
            start: null,
            end: null,
            drag: null
            
        };

        opt = $.extend(defaultSettings, opt);

        if (opt.handle === "") {
            var $handle = $el;
            var customHandle = false;
        } else {
            var $handle = $el.find(opt.handle);
            var customHandle = true;
        }

        $handle.css('cursor', opt.cursor);

        $handle.on("mousedown", function (e) {
            if (opt.checkCondition && !opt.checkCondition()) return;

            is_moving = true;

            $el.addClass('draggable');

            if (customHandle) {
                $handle.addClass('active-handle')
            }

            var dy = $el.offset().top - e.pageY,
                dx = $el.offset().left - e.pageX;

            if (opt.insideParentContainer) {
                var parent = $el.parent();

                var drg_h = $handle.outerHeight(),
                    drg_w = $handle.outerWidth(),
                    drg_dy = $drag.offset().top - e.pageY,
                    drg_dx = $drag.offset().left - e.pageX,
                    parX1 = parseInt($el.offset().left) + parseInt($el.css('padding-left').replace('px', '')),
                    parX2 = parX1 + parW,
                    parY1 = parseInt($el.offset().top) + parseInt($el.css('padding-top').replace('px', '')),
                    parY2 = parY1 + parH;
            }


            var onDrag = function (e) {
                var off_top = e.pageY + dy,
                    off_left = e.pageX + dx,
                    offset = null;

                if (opt.insideParentContainer) {
                    var drg_top = e.pageY + drg_dy,
                        drg_left = e.pageX + drg_dx;

                    if (drg_left < parX1) {
                        off_left += parX1 - drg_left;
                    } else  if (drg_left > parX2 - drg_w) {
                        off_left += parX2 - drg_w - drg_left;
                    }

                    if (drg_top < parY1) {
                        off_top += parY1 - drg_top;
                    } else if (drg_top > parY2 - drg_h) {
                        off_top += parY2 - drg_h - drg_top;
                    }
                }

                if (opt.axis === "x") {
                    offset = {
                        left: off_left
                    };
                } else if (opt.axis === "y") {
                    offset = {
                        top: off_top
                    };
                } else {
                    offset = {
                        left: off_left,
                        top: off_top
                    };
                }

                $el.offset(offset);

                if (opt.drag) opt.drag.call($el, e);

                e.preventDefault();
            }

            var onEndDragging = function (e) {
                active = null;
                
                $el.removeClass('draggable').css("z-index", NORMAL_Z_INDEX);

                if (customHandle) {
                    $handle.removeClass('active-handle');
                }

                $handle.parents().addBack().off('mouseup', onEndDragging).off('mousemove', onDrag);

                if (opt.stop) {
                    opt.stop.call($el);
                }

                e.preventDefault();
            }

            $handle.parents().addBack().on('mouseup', onEndDragging).on('mousemove', onDrag);
            $el.css("z-index", ACTIVE_Z_INDEX);

            if (active) {
                active.$el.css("z-index", NORMAL_Z_INDEX).removeClass('draggable');

                if (active.customHandle) {
                    active.$handle.removeClass('active-handle');
                }

                active.$handle.parents().addBack().off('mouseup', active.onEndDragging).off('mousemove', active.onDrag);
                if (active.end) active.end();

            }

            active = {
                $el: $el,
                customHandle: customHandle,
                onEndDragging: onEndDragging,
                onDrag: onDrag,
                stop: opt.stop,
                $handle: $handle
            }

            if (opt.start) opt.start.call($el, e);

            e.preventDefault(); // disable selection
        })
    }
})(jQuery);

