<?php


class Ecover_For_Dummies_Plugin {
    function __construct($config){
        $this->config = $config;
    }

    function install(){        
        add_action('init', array($this, 'init'));

        add_shortcode($this->config["shortcode"], array($this, 'shortcode' ));

        add_action('edit_form_after_title', array($this, 'edit_form_after_title_callback' ) );

        add_action('save_post_' . $this->config["post_type"], array($this, 'post_save'), 10, 3);
        add_action('manage_' . $this->config["post_type"] . '_posts_custom_column' , array($this, 'manage_posts_custom_column'), 10, 2 );

        add_filter('manage_' . $this->config["post_type"] . '_posts_columns' , array($this, 'manage_columns'));
        add_filter('content_save_pre', array($this, 'content_save_pre'), 10, 1 );
        

        add_action('admin_print_styles-post.php', array($this, 'admin_enqueue_scripts'), 10, 1 );
        add_action('admin_print_styles-post-new.php', array($this, 'admin_enqueue_scripts'), 10, 1 );
    }


    function shortcode($atts){
        $atts = shortcode_atts(
            array (
                'id' => null,
                'debug' => '0',
            ), $atts );

        if (!isset($atts['id'])){
            return 'Parameter "id" for shortcode is missing';
        }

        $id = $atts['id'];

        $json_config = get_post_meta($id, 'config', true);
        if (!$json_config){
            if ($atts["debug"] === "1"){
                return "<p class='shortcode-error'>Ecover $id does not exists</p>";
            } else {
                return "";
            }
        }

        $config = json_decode($json_config, false);

        if (!isset($config->mockup) || (!isset($config->mockup->src) || !isset($config->mockup->width) || !isset($config->mockup->height))) return "";

        $mockupSrc = $config->mockup->src;
        $mockupWidth = $config->mockup->width;
        $mockupHeight = $config->mockup->height;

        $html = '<div id="ecover-' . $id . '" class="ecover">';

        $html .= '<div class="mockup" style="';
        $html .= 'width:' . $mockupWidth . 'px;';
        $html .= 'height:' . $mockupHeight . 'px;';
        $html .= 'background:url(' . esc_url($mockupSrc) . ')' . ' top left / cover no-repeat;';
        $html .= 'padding:0">';

        if (isset($config->cover) && isset($config->cover->transform) && isset($config->cover->src) && isset($config->cover->width) && isset($config->cover->height) && isset($config->dx) && isset($config->dy)){
            $coverTransform = $config->cover->transform;
            $coverBackgroundImage = $config->cover->src;
            $coverWidth = $config->cover->width;
            $coverHeight = $config->cover->height;

            $dx = $config->dx;
            $dy = $config->dy;

            $html .= '<div class="cover" style="';
            $html .= 'width:' . $coverWidth . 'px;';
            $html .= 'height:' . $coverHeight . 'px;';
            $html .= 'top:' . $dy . 'px;';
            $html .= 'left:' . $dx . 'px;';
            $html .= 'transform:' . $coverTransform . ';';
            $html .= 'background-image:url(' . esc_url($coverBackgroundImage) . ');';
            $html .= 'background-size:' . $coverWidth . 'px ' . $coverHeight . 'px;';
            $html .= 'position: relative">';
            $html .= '</div>';
            $html .= '</div>';
        }

        $html .= '</div>';

        return $html;
    }

    function init(){
	    $args = array(
		    'labels'             => $this->config["menu_labels"],
            'description'        => $this->config["description"],
		    'public'             => false,
		    'show_ui'            => true,
		    'show_in_menu'       => true,
		    'show_in_nav_menus'  => false,
		    'query_var'          => true,
		    'rewrite'            => true,
		    'capability_type'    => 'post',
		    'has_archive'        => true,
		    'hierarchical'       => false,
		    'menu_position'      => 12,
		    'menu_icon'			 => $this->config["menu_icon"],
		    'supports'           => array( 'title')
            #"register_meta_box_cb" => array($this, 'meta_box_cb')
	    );

	    register_post_type($this->config["post_type"], $args );
        flush_rewrite_rules(false);

        do_action($this->config["post_type"] . '_init');
    }


    function post_save($post_id, $post, $update){
        if (!isset($_POST['config'])) return;

        $json_config = $_POST['config'];
        $config = json_decode(stripslashes($json_config), false);

        if ($config === null) return;

        $cleaned_config = array();

        if (isset($config->mockup) && isset($config->mockup->src) && isset($config->mockup->width) && isset($config->mockup->height) && isset($config->mockup->x) && isset($config->mockup->y)){
            if (!is_numeric($config->mockup->x) || !is_numeric($config->mockup->y) || !is_numeric($config->mockup->width) || !is_numeric($config->mockup->height)){
                return;
            }

            $cleaned_config["mockup"] = array();
            $cleaned_config["mockup"]["src"] = sanitize_url($config->mockup->src);
            $cleaned_config["mockup"]["x"] = (int)$config->mockup->x;
            $cleaned_config["mockup"]["y"] = (int)$config->mockup->y;
            $cleaned_config["mockup"]["width"] = (int)$config->mockup->width;
            $cleaned_config["mockup"]["height"] = (int)$config->mockup->height;
        }

        if (isset($config->cover) && isset($config->cover->transform) && isset($config->cover->src) && isset($config->cover->width) && isset($config->cover->height) && isset($config->cover->x) && isset($config->cover->y)){
            if (!is_numeric($config->cover->x) || !is_numeric($config->cover->y) || !is_numeric($config->cover->width) || !is_numeric($config->cover->height)){
                return;
            }
            $cleaned_config["cover"] = array();
            $cleaned_config["cover"]["x"] = (int)$config->cover->x;
            $cleaned_config["cover"]["y"] = (int)$config->cover->y;
            $cleaned_config["cover"]["src"] = sanitize_url($config->cover->src);
            $cleaned_config["cover"]["width"] = (int)$config->cover->width;
            $cleaned_config["cover"]["height"] = (int)$config->cover->height;

            # transform = matrix3d(0.28797, 0.0659992, 0, -0.000277659, 0.00383535, 0.306963, 0, -0.000013656, 0, 0, 1, 0, -99.1545, -103.773, 0, 1)
            if (preg_match('/matrix3d\(-?\d+(?:\.\d+)?(?:\s*,\s*-?\d+(?:\.\d+)?){15}\)/', $config->cover->transform) === 0) return;
            $cleaned_config["cover"]["transform"] = $config->cover->transform;

            if (isset($config->cover->corners) && isset($config->cover->corners->topLeft) && isset($config->cover->corners->topLeft->x) && isset($config->cover->corners->topLeft->y) && isset($config->cover->corners->topRight) && isset($config->cover->corners->topRight->x) && isset($config->cover->corners->topRight->y) && isset($config->cover->corners->bottomLeft) && isset($config->cover->corners->bottomLeft->x) && isset($config->cover->corners->bottomLeft->y) && isset($config->cover->corners->bottomRight) && isset($config->cover->corners->bottomRight->x) && isset($config->cover->corners->bottomRight->y)){
                if (!is_numeric($config->cover->corners->topLeft->x) || !is_numeric($config->cover->corners->topLeft->y) || !is_numeric($config->cover->corners->topRight->x) || !is_numeric($config->cover->corners->topRight->y) || !is_numeric($config->cover->corners->bottomLeft->x) || !is_numeric($config->cover->corners->bottomLeft->y) || !is_numeric($config->cover->corners->bottomRight->x) || !is_numeric($config->cover->corners->bottomRight->y)){
                    return;
                }

                $cleaned_config["cover"]["corners"] = array();
                $cleaned_config["cover"]["corners"]["topLeft"] = array("x"=> (int)$config->cover->corners->topLeft->x, "y"=> (int)$config->cover->corners->topLeft->y);
                $cleaned_config["cover"]["corners"]["topRight"] = array("x"=> (int)$config->cover->corners->topRight->x, "y"=> (int)$config->cover->corners->topRight->y);
                $cleaned_config["cover"]["corners"]["bottomLeft"] = array("x"=> (int)$config->cover->corners->bottomLeft->x, "y"=> (int)$config->cover->corners->bottomLeft->y); 
                $cleaned_config["cover"]["corners"]["bottomRight"] = array("x"=> (int)$config->cover->corners->bottomRight->x, "y"=> (int)$config->cover->corners->bottomRight->y);
            }
        }

        if (array_key_exists("mockup", $cleaned_config) && array_key_exists("cover", $cleaned_config) && isset($config->dx) && isset($config->dy)){
            if (!is_numeric($config->dx)) return;
            $cleaned_config["dx"] = (int)$config->dx;

            if (!is_numeric($config->dy)) return;
            $cleaned_config["dy"] = (int)$config->dy;
        }

        update_post_meta($post_id, 'config', addslashes(json_encode($cleaned_config)));
    }

    function manage_columns($columns) {
        $columns['shortcode'] = 'Shortcode';

        return $columns;
    }

    function manage_posts_custom_column( $column_name, $id ) {
        if ( $column_name == 'shortcode' ) {
            echo $this->create_shortcode($id);
        }
    }

    function create_shortcode($id){
        return "[" . $this->config["shortcode"] . " id='$id']";
    }


    function edit_form_after_title_callback($post){
        if ( $post->post_type == $this->config["post_type"]) {
            echo "<input name='config' type='hidden'><div id='ecover_ui_iframe_wrap'></div>";
        }
    }

    // Otra posibilidad: wp_localize_script

    function content_save_pre( $content ) {
        $post = get_post();

        if ($post){
            if ( $post->post_type == $this->config["post_type"]) {
                return $this->create_shortcode($post->ID);
            } else {
                return $content;
            }
        } else {
            return $content;
        }
    }

    function admin_enqueue_scripts() {
        $post = get_post();

        // Usar get_post_type()?
        if ( $post->post_type !== $this->config["post_type"]) return;

        wp_enqueue_media();

        wp_enqueue_style('image_picker', plugins_url('assets/lib/image-picker/image-picker.css', __FILE__), array(), null);
        wp_enqueue_style('thickbox');

        wp_enqueue_script('image_picker', plugins_url('assets/lib/image-picker/image-picker.js', __FILE__), array('jquery'), null);
        wp_enqueue_script('thickbox');
        wp_enqueue_script('iframe_rpc', plugins_url('assets/js/iframe_rpc.js', __FILE__), array(), null);
        wp_enqueue_script('build_ecover_plugin_in_admin', plugins_url('assets/js/build_ecover_plugin_in_admin.js', __FILE__), array('jquery', 'thickbox', 'iframe_rpc'), null);

        $post = get_post();

        $post_type = $post->post_type;
        if ($post_type !== $this->config["post_type"]) return;

        $plugin_data_config = get_post_meta($post->ID, 'config', true);

        if (isset($plugin_data_config)){
            $plugin_data['config'] = json_decode($plugin_data_config, true);
        }

        $wp_scripts = wp_scripts();
        $jquery_url = $wp_scripts->base_url . $wp_scripts->registered['jquery-core']->src;
 
        $mockup_image_url_list = array();

		$mockup_file_names = @scandir( __DIR__ . '/assets/img' );
		if ( ! $mockup_file_names ) {
			trigger_error( "'ecover_for_dummies/assets/img' is not readable", E_USER_NOTICE );
			return;
		}

	    sort( $mockup_file_names );

		foreach ( $mockup_file_names as $mockup_file_name ) {
			if ( '.' === $mockup_file_name[0] ) {
				continue;
			}

            $mockup_image_url_list[] = plugins_url('assets/img/' . $mockup_file_name, __FILE__);
        }

        $iframe_html_template = file_get_contents(__DIR__ . '/assets/index.html.tmpl');

        $iframe_html = strtr($iframe_html_template, array(
            '{image_picker_css_url}' => plugins_url('assets/lib/image-picker/image-picker.css', __FILE__),
            '{style_css_url}' => plugins_url('assets/css/style.css', __FILE__),
            '{materialize_css_url}' => plugins_url('assets/lib/materialize/materialize.css', __FILE__),
            '{page_transitions_css_url}' => plugins_url('assets/lib/page-transitions/css/page-transitions.css', __FILE__),
            '{materialize_js_url}' => plugins_url('assets/lib/materialize/materialize.js', __FILE__),
            '{image_picker_js_url}' => plugins_url('assets/lib/image-picker/image-picker.js', __FILE__),
            '{jquery_draggable_js_url}' => plugins_url('assets/lib/jquery.draggable.js', __FILE__),
            '{page_transitions_js_url}' => plugins_url('assets/lib/page-transitions/js/page-transitions.js', __FILE__),
            '{system_of_linear_equations_js_url}' => plugins_url('assets/js/system_of_linear_equations.js', __FILE__),
            '{perspective_transform_js_url}' => plugins_url('assets/js/perspective_transform.js', __FILE__),
            '{perspective_transform_js_url}' => plugins_url('assets/js/perspective_transform.js', __FILE__),
            '{ecover_builder_js_url}' => plugins_url('assets/js/ecover_builder.js', __FILE__),
            '{iframe_rpc_js_url}' => plugins_url('assets/js/iframe_rpc.js', __FILE__),
            '{jquery_js_url}' => $jquery_url,
            '{mockup_image_url_list}' => json_encode($mockup_image_url_list)
        ));

        $plugin_data['iframe_html'] = $iframe_html;
        $plugin_data['iframe_width'] = $this->config["iframe_width"];
        $plugin_data['iframe_height'] = $this->config["iframe_height"];
        
        wp_localize_script('build_ecover_plugin_in_admin', 'PLUGIN_DATA', $plugin_data);
    }
}

